/*
 * Copyright (c) 1998-2012 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Emil Ong
 */

package com.caucho.jaxb.mapping;

import com.caucho.util.L10N;
import com.caucho.xml.stream.StaxUtil;

import static javax.xml.XMLConstants.*;
import javax.xml.bind.annotation.*;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;
import javax.xml.stream.events.*;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import java.io.IOException;

import java.lang.annotation.*;
import java.lang.reflect.*;

import java.util.Map;
import java.util.logging.Logger;

/**
 *
 **/
public class NilWrapper extends XmlMapping {
  private static final Logger log 
    = Logger.getLogger(NilWrapper.class.getName());
  private static final L10N L = new L10N(NilWrapper.class);
  private static final QName XSI_NIL_NAME 
    = new QName(W3C_XML_SCHEMA_INSTANCE_NS_URI, "nil", "xsi");

  public static final NilWrapper INSTANCE = new NilWrapper();

  private NilWrapper()
  {
  }

  public void write(Marshaller m, XMLStreamWriter out, Object obj)
    throws IOException, XMLStreamException, JAXBException
  {
    StaxUtil.writeAttribute(out, XSI_NIL_NAME, "true");
  }

  public QName getQName(Object obj)
    throws JAXBException
  {
    return XSI_NIL_NAME;
  }

  public void putQNames(Map<QName,XmlMapping> map)
    throws JAXBException
  {
    throw new UnsupportedOperationException();
  }

  public void generateSchema(XMLStreamWriter out)
    throws JAXBException, XMLStreamException
  {
    throw new UnsupportedOperationException();
  }

  public String toString()
  {
    return "NilWrapper[]";
  }
}
