/*
 * Copyright (c) 2001-2008 Caucho Technology, Inc.  All rights reserved.
 *
 * The Apache Software License, Version 1.1
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Caucho Technology (http://www.caucho.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Burlap", "Resin", and "Caucho" must not be used to
 *    endorse or promote products derived from this software without prior
 *    written permission. For written permission, please contact
 *    info@caucho.com.
 *
 * 5. Products derived from this software may not be called "Resin"
 *    nor may "Resin" appear in their names without prior written
 *    permission of Caucho Technology.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL CAUCHO TECHNOLOGY OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @author Emil Ong
 * 
 */

package com.caucho.hmtp
{
  import flash.net.Socket;

  import hessian.io.Hessian2StreamingInput;
  import hessian.io.Hessian2StreamingOutput;

  import com.caucho.bam.*;

  public class ClientBrokerStream implements BamStream {
    private var _socket:Socket;
    private var _input:Hessian2StreamingInput = new Hessian2StreamingInput();
    private var _output:Hessian2StreamingOutput = new Hessian2StreamingOutput();

    public function ClientBrokerStream(socket:Socket):void
    {
      _socket = socket;

      _output.init(_socket);
    }

    public function get input():Hessian2StreamingInput
    {
      return _input;
    }

    public function sendMessage(to:String, from:String, value:Object):void
    {
      if (_output != null) {
        _output.writeObject(new Message(to, from, value));
      }
    }

    public function sendMessageError(to:String, from:String, value:Object,
                                     error:BamError):void
    {
      if (_output != null) {
        _output.writeObject(new MessageError(to, from, value, error));
      }
    }

    public function sendQueryGet(id:Number, 
                                 to:String, from:String, 
                                 query:Object):void
    {
      if (_output != null) {
        _output.writeObject(new QueryGet(id, to, from, query));
      }
    }

    public function sendQuerySet(id:Number, 
                                 to:String, from:String, 
                                 query:Object):void
    {
      if (_output != null) {
        _output.writeObject(new QuerySet(id, to, from, query));
      }
    }

    public function sendQueryResult(id:Number, 
                                    to:String, from:String, 
                                    value:Object):void
    {
      if (_output != null) {
        _output.writeObject(new QueryResult(id, to, from, value));
      }
    }

    public function sendQueryError(id:Number, 
                                   to:String, from:String, 
                                   value:Object, error:BamError):void
    {
      if (_output != null) {
        _output.writeObject(new QueryError(id, to, from, value, error));
      }
    }

    public function close():void
    {
      _socket.close();
      _output = null;
    }
  }
}
